﻿Module OptionalParametersDemo

    Private Const MAXDATEVALUE As Date = #12/31/9999 11:59:59 PM#
    Private Const DEFAULTDATEFORMAT = "MM/dd/yyyy"

    Sub Main()

        'Call with yesterday:
        PrintTimeDeltaOpt(Date.Now.AddDays(-1))

        'Call with day before yesterday, different than output format:
        PrintTimeDeltaOpt(Date.Now.AddDays(-2), DateFormatString:="dddd, dd. MMMM yyyy")

        'It would have also worked this way:
        PrintTimeDeltaOpt(Date.Now.AddDays(-2), , , "dddd, dd. MMMM yyyy")

        'All of it self-selected:
        PrintTimeDeltaOpt(Date.Now.AddDays(-2), Date.Now, "TimeSpan: ",
                          "dddd, dd. MMMM yyyy", "Start time: ", "End time: ", True)

        Console.WriteLine("Press key to end:")
        Console.ReadKey()
    End Sub

    Public Sub PrintTimeDeltaOpt(ByVal Starttime As Date,
                                 Optional ByVal Endtime As Date = MAXDATEVALUE, _
                                 Optional ByVal TimeSpanCaption As String = "Time difference: ", _
                                 Optional ByVal DateFormatString As String = DEFAULTDATEFORMAT, _
                                 Optional ByVal StartTimeCaption As String = "Start time: ", _
                                 Optional ByVal EndTimeCaption As String = "End time: ", _
                                 Optional ByVal PrintDelimitter As Boolean = True)

        'Typical for optional parameters:
        'Here we need to "discover" whether an optional parameter
        'was specified, and if not, replace it with the value
        'we'd actually like.
        If Endtime = MAXDATEVALUE Then
            Endtime = DateTime.Now
        End If

        Dim zeitDifferenz = Endtime - Starttime

        Console.WriteLine(StartTimeCaption & "{0:" & DateFormatString & "}" & vbNewLine &
                          EndTimeCaption & "{1:" & DateFormatString & "}" & vbNewLine &
                          TimeSpanCaption & "{2:dd\.hh\:mm}", Starttime, Endtime, zeitDifferenz)

        If PrintDelimitter Then
            Console.WriteLine(New String("-"c, 75))
            Console.WriteLine()
        End If


    End Sub

End Module

