Imports System.IO

Interface IShopItem
    Property PrintTypeSetting() As PrintType
    ReadOnly Property ItemDescription() As String
    Function ToString() As String
    Property GrossPrice() As Double
End Interface

Module mdlMain

    'The inventory textfile has to be in the application's path.
    Private Filename As String = My.Application.Info.DirectoryPath & "\Inventory.txt"

    Sub Main()

        'StreamReader for reading text files
        Dim locSr As StreamReader
        Dim locList As New DynamicList  ' Dynamic list
        Dim locElements() As String     ' Individual ShopItem elements
        Dim locShopItem As ShopItem     ' One single ShopItem element
        Dim locDisplayMode As PrintType ' Display mode

        'Find out if the text file exists:
        'For simplicity's sake catch possible errors here...
        Try
            locSr = New StreamReader(Filename, System.Text.Encoding.Default)
        Catch ex As Exception
            Console.WriteLine("Error while reading inventory file!" & _
            vbNewLine & ex.Message)
            Console.WriteLine()
            Console.WriteLine("Press key to exit")
            Console.ReadKey()
            Exit Sub
        End Try

        Console.WriteLine("Select (1) for brief and (2) for detailed display")

        Dim locKey As Char = Console.ReadKey.KeyChar
        If locKey = "1"c Then
            locDisplayMode = PrintType.Brief
        Else
            locDisplayMode = PrintType.Detailed
        End If

        Do
            Try

                'Read line
                Dim locLine As String = locSr.ReadLine()

                'Nothing entered, so that's it!
                If String.IsNullOrEmpty(locLine) Then
                    locSr.Close()
                    Exit Do
                End If


                'Overlook semi-colon
                If Not locLine.StartsWith(";") Then

                    'You don't need to declare an explicit Char array
                    'to split the line into the comma-separated elements
                    locElements = locLine.Split(New Char() {";"c})

                    If locElements(FieldOrder.Type) = "1" Then
                        locShopItem = New BookItem(locElements)
                    Else
                        locShopItem = New DVDItem(locElements)
                    End If
                    locList.Add(locShopItem)
                End If

            Catch ex As Exception
                Console.WriteLine("Error while evaluating the inventory file!" & _
                vbNewLine & ex.Message)
                Console.WriteLine()
                Console.WriteLine("Press key to exit")
                Console.ReadKey()
                locSr.Close()
                Exit Sub

            End Try

        Loop

        Dim locGrossAmount As Double = 0

        'Return all elements
        For count As Integer = 0 To locList.Count - 1
            locList(count).PrintTypeSetting = locDisplayMode
            Console.WriteLine(locList(count).ToString())
            locGrossAmount += locList(count).GrossPrice
        Next

        Console.WriteLine()
        Console.WriteLine("-----------------------------")
        Console.WriteLine("Total: " & locGrossAmount.ToString("#,##0.00") & " $")
        Console.WriteLine("-----------------------------")
        Console.WriteLine("-----------------------------")
        Console.WriteLine()
        Console.WriteLine("Press Return to exit")
        Console.ReadLine()

    End Sub

End Module

Enum PrintType
    Brief
    Detailed
End Enum

Enum FieldOrder
    Type
    OrderNumber
    Titel
    AdditionalRemarks1
    GrossPrice
    AdditionalRemarks2
End Enum

MustInherit Class ShopItem
    Implements IShopItem

    Private myTitle As String
    Private myNetPrice As Double
    Private myOrderNumber As String
    Private myPrintTypeSetting As PrintType

    Public Sub New()
        myPrintTypeSetting = PrintType.Detailed
    End Sub

    Public Sub New(ByVal StringArray() As String)
        Title = StringArray(FieldOrder.Titel)
        GrossPrice = Double.Parse(StringArray(FieldOrder.GrossPrice)) / 100
        OrderNumber = StringArray(FieldOrder.OrderNumber)
        PrintTypeSetting = PrintType.Detailed
    End Sub

    Public Property Title() As String
        Get
            Return myTitle
        End Get
        Set(ByVal Value As String)
            myTitle = Value
        End Set
    End Property

    Public Property OrderNumber() As String
        Get
            Return myOrderNumber
        End Get
        Set(ByVal Value As String)
            myOrderNumber = Value
        End Set
    End Property

    Public Property NetPrice() As Double
        Get
            Return myNetPrice
        End Get

        Set(ByVal Value As Double)
            myNetPrice = Value
        End Set

    End Property

    Public ReadOnly Property NetPriceFormatted() As String
        Get
            Return NetPrice.ToString("#,##0.00") + " $"
        End Get
    End Property

    Public MustOverride Property GrossPrice() As Double Implements IShopItem.GrossPrice
    Public ReadOnly Property GrossPriceFormatted() As String
        Get
            Return GrossPrice.ToString("#,##0.00") + " $"
        End Get
    End Property

    Public ReadOnly Property VATAmountFormatted() As String
        Get
            Return (GrossPrice - myNetPrice).ToString("#,##0.00") + " $"
        End Get
    End Property

    Public Overridable ReadOnly Property Description() As String Implements IShopItem.ItemDescription
        Get
            Return OrderNumber & vbTab & Title
        End Get
    End Property

    Public Property PrintTypeSetting() As PrintType Implements IShopItem.PrintTypeSetting
        Get
            Return myPrintTypeSetting
        End Get

        Set(ByVal Value As PrintType)
            myPrintTypeSetting = Value
        End Set
    End Property

    Public Overrides Function ToString() As String Implements IShopItem.ToString
        If PrintTypeSetting = PrintType.Brief Then

            'Shortform: The Description property of the object is used
            'according to the original implementation in the base class
            Return MyClass.Description & vbCr & vbLf & _
            Me.NetPriceFormatted & vbTab & Me.VATAmountFormatted & vbTab & _
            Me.GrossPriceFormatted & vbCr & vbLf
        Else

            'Longform: The overridden and extended Description property
            'of the object itself is used
            Return Me.Description & vbCr & vbLf & _
            Me.NetPriceFormatted & vbTab & Me.VATAmountFormatted & vbTab & _
            Me.GrossPriceFormatted & vbCr & vbLf
        End If
    End Function

End Class

Class BookItem
    Inherits ShopItem

    Protected myAuthor As String

    Public Sub New(ByVal StringArray() As String)
        MyBase.New(StringArray)
        Author = StringArray(FieldOrder.AdditionalRemarks1)
    End Sub

    Public Overridable Property Author() As String
        Get
            Return myAuthor
        End Get
        Set(ByVal Value As String)
            myAuthor = Value
        End Set
    End Property

    Public Overrides Property GrossPrice() As Double
        Get
            Return NetPrice * 1.07
        End Get

        Set(ByVal Value As Double)
            NetPrice = Value / 1.07
        End Set
    End Property

    Public Overrides ReadOnly Property Description() As String
        Get
            Return OrderNumber & vbTab & Title & vbCr & vbLf & "Author: " & Author
        End Get
    End Property

End Class

Class DVDItem
    Inherits ShopItem

    Protected myRunningTime As Integer
    Protected myActor As String

    Public Sub New(ByVal StringArray() As String)
        MyBase.New(StringArray)
        RunningTime = Integer.Parse(StringArray(FieldOrder.AdditionalRemarks1))
        Actor = StringArray(FieldOrder.AdditionalRemarks2)
    End Sub

    Public Overridable Property RunningTime() As Integer
        Get
            Return myRunningTime
        End Get
        Set(ByVal Value As Integer)
            myRunningTime = Value
        End Set
    End Property

    Public Overridable Property Actor() As String
        Get
            Return myActor
        End Get
        Set(ByVal Value As String)
            myActor = Value
        End Set
    End Property

    Public Overrides ReadOnly Property Description() As String
        Get
            Return OrderNumber & vbTab & Title & vbCr & vbLf & "Running time: " & myRunningTime & " Min." & vbCr & vbLf & "Main actor: " & Actor
        End Get
    End Property

    Public Overrides Property GrossPrice() As Double
        Get
            Return NetPrice * 1.19
        End Get

        Set(ByVal Value As Double)
            NetPrice = Value / 1.19
        End Set
    End Property
End Class

Class DynamicList

    Private myStepIncreaser As Integer = 4
    Private myCurrentArraySize As Integer
    Private myCurrentCounter As Integer
    Private myArray() As IShopItem

    Sub New()
        myCurrentArraySize = myStepIncreaser
        ReDim myArray(myCurrentArraySize)
    End Sub

    Sub Add(ByVal Item As IShopItem)

        'Check whether the current array limit has been reached.
        If myCurrentCounter = myCurrentArraySize - 1 Then

            'Create new arry with more storage,
            'and copy elements there.
            'New array becomes larger:
            myCurrentArraySize += myStepIncreaser

            'Create temporary array.
            Dim locTempArray(myCurrentArraySize - 1) As IShopItem

            'Copy elements.
            'Important: You must take care of,
            'copying yourself, unlike in VB6 or VBA!
            For locCount As Integer = 0 To myCurrentCounter
                locTempArray(locCount) = myArray(locCount)
            Next

            'Assign temporary array to member array.
            myArray = locTempArray
        End If

        'Save element in the array
        myArray(myCurrentCounter) = Item

        'Increase pointer to the next element.
        myCurrentCounter += 1
    End Sub

    'Returns the number of the existing elements.
    Public Overridable ReadOnly Property Count() As Integer
        Get
            Return myCurrentCounter
        End Get
    End Property

    'Permits assignment.
    Default Public Overridable Property Item(ByVal Index As Integer) As IShopItem
        Get
            Return myArray(Index)
        End Get
        Set(ByVal Value As IShopItem)
            myArray(Index) = Value
        End Set
    End Property
End Class