Interface IMustHaveValue
    Property Value() As Integer
End Interface

Module mdlMain

    Sub Main()
        Dim AValueType As New ValueType(10)
        Dim AReferenceType As New ReferenceType(10)

        AValueType.Value = 20
        Console.WriteLine(AValueType.Value) ' 20 -> changed directly!
        ChangeValueType(AValueType)

        Console.WriteLine(AValueType.Value) ' 20 -> changed in the stack copy
        ChangeReferenceType(AReferenceType)

        Console.WriteLine(AReferenceType.Value) ' 30 -> changed on the Managed Heap
        Dim AnInterface As IMustHaveValue = AValueType

        ChangeViaInterface(AnInterface)
        Console.WriteLine(AnInterface.Value) ' 40, changed on the Managed Heap

        ' 20, but have nothing to do with each other
        Console.WriteLine(AValueType.Value)

        Console.WriteLine()
        Console.WriteLine("Press key to exit")
        Console.ReadLine()
    End Sub

    'Changes the property of the value type.
    Sub ChangeValueType(ByVal AValueType As ValueType)
        AValueType.Value = 30
    End Sub

    'Changes the property of the reference type.
    Sub ChangeReferenceType(ByVal AReferenceType As ReferenceType)
        AReferenceType.Value = 30
    End Sub

    'Changes the property via the interface.
    Sub ChangeViaInterface(ByVal AnInterface As IMustHaveValue)
        AnInterface.Value = 40
    End Sub
End Module

'Test class of the value type
Structure ValueType

    Implements IMustHaveValue

    Dim myValue As Integer

    Sub New(ByVal Value As Integer)
        myValue = Value
    End Sub

    Property Value() As Integer Implements IMustHaveValue.Value

        Get
            Return myValue
        End Get

        Set(ByVal Value As Integer)
            myValue = Value
        End Set

    End Property

End Structure

'Test class of the reference type
Class ReferenceType
    Implements IMustHaveValue
    Dim myValue As Integer

    Sub New(ByVal Value As Integer)
        myValue = Value
    End Sub

    Property Value() As Integer Implements IMustHaveValue.Value
        Get
            Return myValue
        End Get
        Set(ByVal Value As Integer)
            myValue = Value
        End Set
    End Property
End Class

Class ValueList
    Inherits ArrayList

    Shadows Sub Add(ByVal Value As ValueType)
        MyBase.Add(Value)
    End Sub

    Default Shadows Property Item(ByVal Index As Integer) As ValueType
        Get
            Return DirectCast(MyBase.Item(Index), ValueType)
        End Get
        Set(ByVal Value As ValueType)
            MyBase.Item(Index) = Value
        End Set
    End Property
End Class

Class VerweisList
    Inherits ArrayList

    Shadows Sub Add(ByVal Value As ReferenceType)
        MyBase.Add(Value)
    End Sub

    Default Shadows Property Item(ByVal Index As Integer) As ReferenceType
        Get
            Return DirectCast(MyBase.Item(Index), ReferenceType)
        End Get
        Set(ByVal Value As ReferenceType)
            MyBase.Item(Index) = Value
        End Set
    End Property
End Class