Public Class frmMain

    Private WithEvents myTimer As Timer
    Private WithEvents myAlarmTrigger As AlarmTrigger

    Private myCurrentWind As String

    'Alarm length in 500
    Private myAlarmLength As Integer
    Private myCurrentColor As Color
    Private myAlarmDauer As Integer = 50
    Private myAlarmDownCounter As Integer
    Private myAlarmStatus As Boolean

    Sub New()

        InitializeComponent()

        myTimer = New Timer()
        myTimer.Interval = 500
        myTimer.Start()
        myCurrentColor = Color.White
    End Sub

    'Event handler which is called when the alarm class 
    'signals the alarm because a certain time has been reached.
    Private Sub myAlarmgeber_Alarm(ByVal Sender As Object, ByVal e As AlarmEventArgs) Handles myAlarmTrigger.Alarm

        'Alarm clock rings:
        myAlarmStatus = True

        'And it rings for this amount of time:
        myAlarmDownCounter = myAlarmDauer

        'We should be able to turn this off:
        btnOff.Enabled = True

        'And it should ring at the same time tomorrow.
        e.Reset = True
    End Sub

    'Event handler that is triggered
    'If the contents of the Picturebox going to be redrawn.
    Private Sub picAlarmclock_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) _
                    Handles picAlarmClock.Paint

        If myAlarmTrigger IsNot Nothing AndAlso myAlarmTrigger.AlarmActivated Then
            DrawClock(e.Graphics, Date.Now, myAlarmTrigger.TimeOfAlarm, myCurrentColor)
        Else
            DrawClock(e.Graphics, Date.Now, myCurrentColor)
        End If
    End Sub

    'Being called, when the user as resized the window in any way.
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        'Don't forget to call the original method, when overwriting
        'event raising methods!!
        MyBase.OnResize(e)

        'Paint the content of the picturebox, if the size has changed.
        picAlarmClock.Invalidate()

    End Sub

    'Event handling method is triggered
    'when the timer fires. This happens
    'every 500 milliseconds, at which point we redraw
    'the complete clocktaking background blinking
    'into consideration if the alarm clock is going off.
    Private Sub myTimer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles myTimer.Tick

        'Is the alarm going off?
        If myAlarmStatus Then

            'Yes, change colors every 500 ms
            If myCurrentColor = Color.White Then
                myCurrentColor = Color.Red
            Else
                myCurrentColor = Color.White
            End If

            'Reduce alarm duration counter
            myAlarmDownCounter -= 1
            If myAlarmDownCounter = 0 Then
                'Switch off alarm when it has run out.
                SwitchOffAlarm(True)
            End If
        End If

        'Redraw entire clock in any case
        picAlarmClock.Invalidate()
    End Sub

    'Switches of the alarm.
    Private Sub SwitchOffAlarm(ByVal reactivate As Boolean)

        'Reset status of alarm.
        myAlarmStatus = False

        'Deactivate "Snooze" Button
        btnOff.Enabled = False

        'Background is reset to white.
        myCurrentColor = Color.White
    End Sub

    '"Snooze" has been pressed...
    Private Sub btnOff_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOff.Click
        'so switch of the alarm, but (true) reschedule the 
        'alarm time for the next day.
        SwitchOffAlarm(True)
    End Sub

    'Text of AlarmTime has been changed.
    Private Sub mtbAlarmtime_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles mtbAlarmTime.TextChanged
        ''Switch of the alarm.
        chkActivateAlarm.Checked = False
    End Sub

    'The alarm clock is being switched on or off completely.
    Private Sub chkAlarmAktivieren_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkActivateAlarm.CheckedChanged
        'Here, it is switched on...
        If chkActivateAlarm.Checked Then
            Dim locAlarm As Date
            'Get the alarm time from the textbox
            If Date.TryParse(mtbAlarmTime.Text, locAlarm) Then

                'Instanciate new AlarmTrigger object, whose events we're
                'listening to, since it was declared with WithEvents.
                myAlarmTrigger = New AlarmTrigger(locAlarm, True)

                'Redraw the clock to show the alarm time 
                'as a red line on the clock's face.
                picAlarmClock.Invalidate()
            Else
                'Whoopsy! Invalid time format - TryParse
                'failed.
                MessageBox.Show("Please check the time format!")
            End If
        Else
            'Alarm is switched off.
            SwitchOffAlarm(False)
            'We need no events, there is nothing to be alarmed about! ;-)
            myAlarmTrigger = Nothing
        End If
    End Sub

    Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
        Me.Close()
    End Sub

#Region "Painting by numbers... ;-)"

    Private Sub DrawClock(ByVal g As Graphics, ByVal time As Date, ByVal background As Color)
        DrawClock(g, time, Nothing, background)
    End Sub

    Private Sub DrawClock(ByVal g As Graphics, ByVal Time As Date, ByVal selectionTime As Nullable(Of Date), ByVal background As Color)

        'Create Brushes and Pens
        Dim locPenWidth As Single = 3
        Dim locHalfPenWidth As Single = locPenWidth / 2
        Dim locCenter As PointF
        Dim locSize As SizeF

        Dim locClockDimension As RectangleF = New RectangleF( _
            locHalfPenWidth, locHalfPenWidth, _
            (picAlarmClock.ClientSize.Width - 1) - locPenWidth, _
            (picAlarmClock.ClientSize.Height - 1) - locPenWidth)

        'Find the center
        locCenter.X = picAlarmClock.ClientSize.Width / 2.0F
        locCenter.Y = picAlarmClock.ClientSize.Height / 2.0F

        'Size without the clock's border
        locSize.Width = locClockDimension.Width / 2 - 5
        locSize.Height = locClockDimension.Height / 2 - 5

        'Enable AntiAlias
        g.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias

        g.Clear(background)

        'Paint outer circle of the clock.
        g.DrawEllipse(New Pen(Color.Black, locPenWidth), locClockDimension)

        'Paint the clock's face.
        Dim locOffset As Single
        For locAngle As Single = 0 To 359 Step 30
            If locAngle = 0 Or locAngle = 90 Or locAngle = 180 Or locAngle = 270 Then
                locOffset = 20
            Else
                locOffset = 10
            End If

            Dim locPcStart As New PolarCoordinate(locCenter, locSize.Width - 5, locSize.Height - 5, locAngle)
            Dim locPcEnd As New PolarCoordinate(locCenter, locSize.Width - locOffset, _
                                locSize.Height - locOffset, locAngle)
            g.DrawLine(Pens.Black, locPcStart.Cartesian.X, _
                                  locPcStart.Cartesian.Y, _
                                  locPcEnd.Cartesian.X, _
                                  locPcEnd.Cartesian.Y)
        Next

        'Paint the clock's hands

        'Parameter for hour hand
        Dim locHourHandAngle As Single = Time.Hour
        If locHourHandAngle > 12 Then
            locHourHandAngle -= 12
        End If
        locHourHandAngle *= 30.0F
        locHourHandAngle += (Time.Minute / 2.0F)
        locHourHandAngle -= 90


        'Parameter for the alarm tag
        Dim locAlarmTagAngle As Single
        If selectionTime.HasValue Then
            locAlarmTagAngle = selectionTime.Value.Hour
            If locAlarmTagAngle > 12 Then
                locAlarmTagAngle -= 12
            End If
            locAlarmTagAngle *= 30.0F
            locAlarmTagAngle += (selectionTime.Value.Minute / 2.0F)
            locAlarmTagAngle -= 90
        End If

        'Parameter for the minute hand
        Dim locMinuteHandAngle As Single = Time.Minute
        locMinuteHandAngle *= 6
        locMinuteHandAngle -= 90

        'Parameter for the second hand
        Dim locSecondHandAngle As Single = Time.Second
        locSecondHandAngle *= 6
        locSecondHandAngle -= 90

        'First draw the hour hand...
        Dim locHandStart As New PolarCoordinate(locCenter, 0, 0, locHourHandAngle)
        Dim locHandEnd As New PolarCoordinate(locCenter, locSize.Width / 2, locSize.Height / 2, locHourHandAngle)
        g.DrawLine(New Pen(Color.Blue, 5), locHandStart.Cartesian.X, _
                                           locHandStart.Cartesian.Y, _
                                           locHandEnd.Cartesian.X, _
                                           locHandEnd.Cartesian.Y)

        '... then the minute hand ...
        locHandStart = New PolarCoordinate(locCenter, 0, 0, locMinuteHandAngle)
        locHandEnd = New PolarCoordinate(locCenter, locSize.Width - 25, locSize.Height - 25, locMinuteHandAngle)
        g.DrawLine(New Pen(Color.Blue, 3), locHandStart.Cartesian.X, _
                                           locHandStart.Cartesian.Y, _
                                           locHandEnd.Cartesian.X, _
                                           locHandEnd.Cartesian.Y)

        '... then the second hand ...
        locHandStart = New PolarCoordinate(locCenter, 0, 0, locSecondHandAngle)
        locHandEnd = New PolarCoordinate(locCenter, locSize.Width - 20, locSize.Height - 20, locSecondHandAngle)
        g.DrawLine(New Pen(Color.Black, 1), locHandStart.Cartesian.X, _
                                           locHandStart.Cartesian.Y, _
                                           locHandEnd.Cartesian.X, _
                                           locHandEnd.Cartesian.Y)

        '... and if there is an alarm time to tag, draw that as well!
        If selectionTime.HasValue Then
            locHandStart = New PolarCoordinate(locCenter, locSize.Width / 4 * 3, locSize.Height / 4 * 3, locAlarmTagAngle)
            locHandEnd = New PolarCoordinate(locCenter, locSize.Width - 5, locSize.Height - 5, locAlarmTagAngle)
            g.DrawLine(New Pen(Color.Red, 5), locHandStart.Cartesian.X, _
                                               locHandStart.Cartesian.Y, _
                                               locHandEnd.Cartesian.X, _
                                               locHandEnd.Cartesian.Y)
        End If
    End Sub
#End Region
End Class
