﻿'Defines a new variable type for saving methods
'which fulfill this signature:
Delegate Function DrawRectangleDelegate(ByVal xLeft As Double,
        ByVal yTop As Double, ByVal xRight As Double, ByVal yBottom As Double) As Rectangle

Class MainWindow

    'Declare delegate variable and define with the function for determining
    'an "angular" rectangle (we renamed it differing from the printed code in the book).
    Private rectangleGetter As DrawRectangleDelegate = AddressOf angularCorners

    'Click draws the figure from many rectangles.
    Private Sub DrawButton_Click(ByVal sender As System.Object,
                                 ByVal e As System.Windows.RoutedEventArgs) Handles DrawButton.Click
        Redraw()
    End Sub

    'Switch between rounded and angular corners via checkbox.
    Private Sub RoundedCornersCheckBox_ToggleCheck(ByVal sender As System.Object,
                            ByVal e As System.Windows.RoutedEventArgs) _
            Handles RoundedCornersCheckBox.Checked, RoundedCornersCheckBox.Unchecked

        If RoundedCornersCheckBox.IsChecked Then
            rectangleGetter = AddressOf RoundedCorners
        Else
            rectangleGetter = AddressOf AngularCorners
        End If
        Redraw()

    End Sub

    'Redraws the graphic.
    Private Sub Redraw()
        Dim s As New Rectangle()
        DrawAreaCanvas.Children.Clear()

        'Fan out a few rectangles, i.e. turn them during drawing...
        For c = 0 To 120 Step 3

            '...and move each next one a bit to the
            'right and down.
            Dim x1, y1 As Double
            x1 = 10 + c * 3
            y1 = c \ 2

            'Here we find out the rectangle object via the delegate
            'which contains a different "address" depending on whether
            'the checkbox was activated or not.
            Dim recToDraw = rectangleGetter(x1, y1, x1 + 300, y1 + 200)

            'Determining the position in WPF on the canvas only works
            'via attached properties.
            Canvas.SetLeft(recToDraw, DirectCast(recToDraw.Tag, Point).X)
            Canvas.SetTop(recToDraw, DirectCast(recToDraw.Tag, Point).Y)

            'We turn the rectangle c degrees; the center of rotation
            'has been moved a bit to the left and up.
            recToDraw.LayoutTransform = New RotateTransform(c, -100, -50)
            'Adding to the children collection on the canvas
            'created the screen display.
            DrawAreaCanvas.Children.Add(recToDraw)
        Next
    End Sub

    'Creates a WPF rectangle object with normal corners at the specified coordinates...
    Private Function AngularCorners(ByVal x1 As Double, ByVal y1 As Double,
                                    ByVal x2 As Double, ByVal y2 As Double) As Rectangle

        Return New Rectangle() With {.Tag = New Point(x1, y1),
                        .Width = x2 - x1,
                        .Height = y2 - y1,
                        .Stroke = New SolidColorBrush(Colors.Black)}

    End Function

    '...and this method creates on with rounded corners.
    Private Function RoundedCorners(ByVal x1 As Double, ByVal y1 As Double,
                                    ByVal x2 As Double, ByVal y2 As Double) As Rectangle

        Return New Rectangle() With {.Tag = New Point(x1, y1),
                                     .Width = x2 - x1,
                                     .Height = y2 - y1,
                                     .RadiusX = 30,
                                     .RadiusY = 30,
                                     .Stroke = New SolidColorBrush(Colors.Black)}
    End Function

End Class
