﻿Module MainModule

    Dim Contacts(0 To 100) As Contact

    Sub Main()

        'Just for a change...a variation of colors in the Console Window! :-)
        Console.BackgroundColor = ConsoleColor.White
        Console.ForegroundColor = ConsoleColor.Black
        Console.Clear()

        'Generate random addresses.
        Console.Write("Generating random addresses ... ")
        GenerateRandomAddresses()
        Console.WriteLine("Ready!")

        'Print the first 10 addresses.
        PrintAddresses(0, 10)

        'Sort the addresses by last name
        Console.WriteLine()
        Console.Write("Addresses are being sorted by last name... ")
        Dim compDelegate As ComparerDelegate = AddressOf ContactLastNameComparison

        SortAddresses(compDelegate)
        Console.WriteLine("Done!")
        Console.WriteLine()
        'Output the first 10 random addresses
        PrintAddresses(0, 10)
        'Sort the addresses by City
        Console.WriteLine()
        Console.Write("Addresses are being sorted by city name... ")
        compDelegate = AddressOf ContactCityComparison
        SortAddresses(compDelegate)
        Console.WriteLine("Done!")
        Console.WriteLine()

        'Print the first 10 addresses again:
        PrintAddresses(0, 10)

        'Wait for the program to be finished.
        Console.WriteLine()
        Console.WriteLine("Press any key to quit...")
        Console.ReadKey()

    End Sub

    Function ContactLastNameComparison(ByVal k1 As Contact, ByVal k2 As Contact) As Integer
        Return String.Compare(k1.LastName, k2.LastName)
    End Function

    Function ContactCityComparison(ByVal k1 As Contact, ByVal k2 As Contact) As Integer
        Return String.Compare(k1.City, k2.City)
    End Function


    Sub PrintAddresses(ByVal from As Integer, ByVal [to] As Integer)
        For c As Integer = from To [to]
            Console.WriteLine(c.ToString("000") & ": " & Contacts(c).ContactText())
        Next
    End Sub

    'Defines the type ComparerDelegate which expects two parameters of the type
    'Contact and returns an Integer.
    '-1: first contact is less
    '0: contact is the same
    '1: first contact is larger

    'Why a contact is actually larger or less is regulated by the method
    'whose address is saved in the delegate.
    'Therefore the sorting algorithm can be used more flexibly.
    Public Delegate Function ComparerDelegate(ByVal k1 As Contact, ByVal k2 As Contact) As Integer

    'cDel is a delegate variable of the type ComparerDelegate.
    Sub SortAddresses(ByVal cDel As ComparerDelegate)

        Dim numberOfElements As Integer = 101
        Dim outerCounter As Integer
        Dim innerCounter As Integer
        Dim delta As Integer
        Dim tempContact As Contact
        delta = 1
        'Determine the largest value of the distance order
        Do
            delta = 3 * delta + 1
        Loop Until delta > numberOfElements
        Do
            'Cancellation criteria—therefore count down
            delta \= 3
            'Shellsort's kernel algorithmus
            For outerCounter = delta To numberOfElements - 1
                tempContact = Contacts(outerCounter)
                innerCounter = outerCounter
                Do
                    'Delegate variables can either be called directly via the name
                    'like methods or with the Invoke methode.
                    If cDel.Invoke(tempContact, Contacts(innerCounter - delta)) = 1 OrElse _
                    cDel(tempContact, Contacts(innerCounter - delta)) = 0 Then Exit Do
                    Contacts(innerCounter) = Contacts(innerCounter - delta)
                    innerCounter = innerCounter - delta
                    If (innerCounter <= delta) Then Exit Do
                Loop
                Contacts(innerCounter) = tempContact
            Next
        Loop Until delta = 0
    End Sub

    Public Sub GenerateRandomAddresses()

        Dim locRandom As New Random(Now.Millisecond)

        Dim locLastNames As String() = {"Heckhuis", "Löffelmann", "Jones", "Lowel", _
                    "Ardelean", "Beckham", "Baur", "Picard", "Trouv", "Feigenbaum", _
                    "Miller", "Wallace", "Merkel", "Spooner", "Spoonman", "Huffman", _
                    "Rode", "Trouw", "Schindler", "Brown", "Walker", "Cruise", _
                    "Tinoco", "O'Reilly", "O'Donnell", "Ó Briain", "Russel", "English", _
                    "Clarke", "Schumacher"}

        Dim locStreetNames As String() = {"Wiedenbrückerstr.", "Stauffenberg Ave.", "Broadway", "Parkstr.", _
                         "Kurgartenweg", "Alter Postweg", "Long Turnpike", "Zzyzx Rd.", "Main Street", _
                         "Streetway", "Postplatz", "Beamer Place", "Mercedes Way", "Porsche Drive", _
                         "Weidering", "One Way", "Endof Rd.", "Gotlost Way", "Satnav Rd."}

        Dim locFirstNames As String() = {"Jürgen", "Gabriele", "Dianne", "Katrin", "Jack", _
                    "Arnold", "Christian", "Frank", "Curt", "Peter", "Anne", "Anja", _
                    "Theo", "Bob", "Katrin", "Guido", "Barbara", "Bernhard", "Margarete", _
                    "Alfred", "Melanie", "Britta", "José", "Thomas", "Dara", "Klaus", "Axel", _
                    "Gabby", "Gareth", "Bob", "Denise", "Kristen"}

        Dim locCities As String() = {"Bellevue", "Dortmund", "Lippstadt", "Redmond", _
                    "Los Angeles", "Las Vegas", "Seattle", "New York", "Berlin", "Bielefeld", _
                    "Braunschweig", "Munich", "Cologne", "Hamburg", _
                    "Bad Waldliesborn", "Bremen", "Encinitas", "Anaheim"}

        For i As Integer = 0 To 100

            Dim adr As New Contact

            adr.LastName = locLastNames(locRandom.Next(locLastNames.Length - 1))
            adr.FirstName = locFirstNames(locRandom.Next(locFirstNames.Length - 1))
            adr.City = locCities(locRandom.Next(locCities.Length - 1))
            adr.ZIP = locRandom.Next(99999).ToString("00000")
            Contacts(i) = adr
        Next
    End Sub
End Module
