﻿Module Enums

    Public Enum ContactCategory As Short
        Family
        'We have to use the brackets here for the english localization since
        'the translation of the German word "freund" becomes a VB keyword.
        [Friend]
        Acquaintance
        Colleague
        BusinessPartner
        Client
        Supplier
        ToAvoid
        Company
        ContactPersonAtCompanyContact
    End Enum

    Sub main()

        'EnumFlags()

        Dim locContacts As ContactCategory
        Dim locShort As Short

        locContacts = ContactCategory.BusinessPartner

        'Type conversion from underlying data type of enum...
        locShort = locContacts
        locShort = ContactCategory.Company

        '...and vice versa, not difficult either:
        locContacts = CType(locShort, ContactCategory)

        Console.WriteLine(locContacts)

        'Retrieves the name of the underlying basic data type of an enumeration.
        Console.WriteLine([Enum].GetUnderlyingType(GetType(ContactCategory)).Name)

        'Retrieves the type name using an enumeration variable.
        Console.WriteLine([Enum].GetUnderlyingType(locContacts.GetType).Name)

        'Converts into a string.
        Console.WriteLine(locContacts.ToString())

        'Converting back to an Enum element from a string.
        Dim locString As String = "BusinessPartner"
        locContacts = DirectCast([Enum].Parse(GetType(ContactCategory), locString), ContactCategory)

        'Since .NET Framework 4.0 this became more easy:
        If [Enum].TryParse(locString, locContacts) Then
            Console.WriteLine("Enum value BusinessPartner was recognized and converted!")
        End If
        Console.ReadLine()

    End Sub

    Sub EnumFlags()

        Dim locContacts As ContactCategories
        locContacts = ContactCategories.[Friend] Or ContactCategories.BusinessPartner
        'Console.WriteLine(locKontakte)
        'Console.WriteLine(locKontakte.ToString())

        'Attention using Flags! Bits must be masked!
        'This query provides the wrong result!
        If locContacts = ContactCategories.BusinessPartner Then
            Console.WriteLine("You are a BusinessPartner")
        Else
            Console.WriteLine("You are not a BusinessPartner")
        End If

        'This is the correct version. This query provides the correct result:
        If (locContacts And ContactCategories.Friend) = ContactCategories.Friend Then
            Console.WriteLine("You are a Friend")
        Else
            Console.WriteLine("You are not a Friend")
        End If

        'And this is how combination queries work:
        If locContacts = (ContactCategories.Friend Or ContactCategories.BusinessPartner) Then
            Console.WriteLine("You are a Friend and a BusinessPartner")
        End If

        'With .NET-Framework 4.0 even this works:
        If locContacts.HasFlag(ContactCategories.[Friend]) Then
            Console.WriteLine("You are a Friend")
        Else
            Console.WriteLine("You are not a Friend")
        End If

        'This is how combination of flags work:
        If locContacts = (ContactCategories.[Friend] Or ContactCategories.BusinessPartner) Then
            Console.WriteLine("You're a friend and a business partner!")
        End If

        Console.ReadLine()

    End Sub

End Module

<Flags()> _
Public Enum ContactCategories
    No = 0
    Family = 1
    [Friend] = 2
    Acquaintance = 4
    Colleague = 8
    BusinessPartner = 16
    Client = 32
    Supplier = 64
    ToAvoid = 128
    Company = 256
    ContactPersonAtCompanyContact = 512
End Enum
