Module DynamicListDemo

    Private Const SAMPLESIZE = 200000

    Sub Main()

        'Uncomment the following lines
        'to experiment with the second example:

        'Example2()
        'Return

        Dim locDynamicList As New DynamicList
        Dim locRandom As New Random(Now.Millisecond)

        Console.WriteLine("Creating {0} random Double elements...", SAMPLESIZE)
        Dim locTimer = Stopwatch.StartNew
        For count As Integer = 1 To SAMPLESIZE
            locDynamicList.Add(locRandom.NextDouble * locRandom.Next)
        Next

        locTimer.Stop()
        Console.WriteLine("...{0:#,##0} milliseconds!",
        locTimer.ElapsedMilliseconds)
        Console.ReadLine()

    End Sub

    Sub Example2()

        Dim locDynamicList As New ArrayList
        Dim locRandom As New Random(Now.Millisecond)
        Console.WriteLine("Creating {0} random Double elements...", SAMPLESIZE)

        Dim locTimer = Stopwatch.StartNew
        For count As Integer = 1 To SAMPLESIZE
            locDynamicList.Add(locRandom.NextDouble * locRandom.Next)
        Next

        locTimer.Stop()
        Console.WriteLine("...{0:#,##0} milliseconds!", locTimer.ElapsedMilliseconds)
        Console.ReadLine()
    End Sub

End Module

Class DynamicList
    Implements IEnumerable

    Private myStepIncreaser As Integer
    Private myCurrentArraySize As Integer
    Private myCurrentCounter As Integer
    Private myArray() As Object

    Sub New()

        MyClass.New(16)

    End Sub

    Sub New(ByVal StepIncreaser As Integer)

        myStepIncreaser = StepIncreaser
        myCurrentArraySize = myStepIncreaser
        ReDim myArray(myCurrentArraySize)

    End Sub

    Sub Add(ByVal Item As Object)

        'Check whether the current array limit has been reached
        If myCurrentCounter = myCurrentArraySize - 1 Then

            'Create new array with more memory,
            'and copy elemente into it.
            'New array becomes larger:
            myCurrentArraySize += myStepIncreaser

            'Create temporary array
            Dim locTempArray(myCurrentArraySize - 1) As Object

            'Copy elements
            'Important: Unlike in VB6, you have to take
            'care of the copying yourself!
            Array.Copy(myArray, locTempArray, myArray.Length)

            'Assign the temporary array to the member array
            myArray = locTempArray

            'Next time more elements are reserved!
            myStepIncreaser *= 2
        End If

        'Save the element in the array
        myArray(myCurrentCounter) = Item

        'Increase the pointer to the next element
        myCurrentCounter += 1
    End Sub

    'Returns the number of the existing elements
    Public Overridable ReadOnly Property Count() As Integer
        Get
            Return myCurrentCounter
        End Get
    End Property

    'Permits assignments and queries
    Default Public Overridable Property Item(ByVal Index As Integer) As Object
        Get
            Return myArray(Index)
        End Get

        Set(ByVal Value As Object)
            myArray(Index) = Value
        End Set
    End Property

    'Returns the enumerator of the base (to the array)
    Public Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        Return myArray.GetEnumerator
    End Function

End Class
