Imports System.Collections.ObjectModel
Imports System.XML.Serialization
Imports System.IO

<Serializable()>
Public Class Addresses
    Inherits List(Of Address)

    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    ''' <remarks></remarks>
    Sub New()
        MyBase.New()
    End Sub

    ''' <summary>
    ''' Creates a new instance of this class based on an existing collection.
    ''' </summary>
    ''' <param name="collection">The collection the new instance should be based on.</param>
    ''' <remarks></remarks>
    Sub New(ByVal collection As ICollection(Of Address))
        MyBase.New(collection)
    End Sub

    ''' <summary>
    ''' Serializes every item of this collection in to a file in XML format.
    ''' </summary>
    ''' <param name="fileName">Filename of the XML file.</param>
    ''' <remarks></remarks>
    Sub XMLSerialize(ByVal fileName As String)
        Dim locXmlWriter As New XmlSerializer(GetType(Addresses))
        Dim locXmlFile As New StreamWriter(fileName)
        locXmlWriter.Serialize(locXmlFile, Me)
        locXmlFile.Flush()
        locXmlFile.Close()
    End Sub

    ''' <summary>
    ''' Generates a new instance of this collection class from the passed XML file.
    ''' </summary>
    ''' <param name="fileName">Name of the XML file.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function XmlDeserialize(ByVal fileName As String) As Addresses
        Dim locXmlReader As New XmlSerializer(GetType(Addresses))
        Dim locXmlFile As New StreamReader(fileName)
        Return CType(locXmlReader.Deserialize(locXmlFile), Addresses)
    End Function

    ''' <summary>
    ''' Returns an instance with count random addresses.
    ''' </summary>
    ''' <param name="count">Numbaer of random addresses to be created.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function RandomAddresses(ByVal count As Integer) As List(Of Address)

        Dim tmpListOfAddresses As New List(Of Address)
        Dim tmpRandom As New Random(Now.Millisecond)

        Dim tmpLastNames As String() = {"Heckhuis", "Lffelmann", "Jones", "Lowel", _
                    "Ardelean", "Beckham", "Baur", "Picard", "Trouv", "Feigenbaum", _
                    "Miller", "Wallace", "Merkel", "Spooner", "Spoonman", "Huffman", _
                    "Rode", "Trouw", "Schindler", "Brown", "Walker", "Cruise", _
                    "Tinoco", "O'Reilly", "O'Donnell", " Briain", "Russel", "English", _
                    "Clarke", "Schumacher"}

        Dim tmpStreetNames As String() = {"Wiedenbrckerstr.", "Stauffenberg Ave.", "Broadway", "Parkstr.", _
                         "Kurgartenweg", "Alter Postweg", "Long Turnpike", "Zzyzx Rd.", "Main Street", _
                         "Streetway", "Postplatz", "Beamer Place", "Mercedes Way", "Porsche Drive", _
                         "Weidering", "One Way", "Endof Rd.", "Gotlost Way", "Satnav Rd."}

        Dim tmpFirstNames As String() = {"Jrgen", "Gabriele", "Dianne", "Katrin", "Jack", _
                    "Arnold", "Christian", "Frank", "Curt", "Peter", "Anne", "Anja", _
                    "Theo", "Bob", "Katrin", "Guido", "Barbara", "Bernhard", "Margarete", _
                    "Alfred", "Melanie", "Britta", "Jos", "Thomas", "Dara", "Klaus", "Axel", _
                    "Gabby", "Gareth", "Bob", "Denise", "Kristen"}

        Dim tmpCities As String() = {"Bellevue", "Dortmund", "Lippstadt", "Redmond", _
                    "Los Angeles", "Las Vegas", "Seattle", "New York", "Berlin", "Bielefeld", _
                    "Braunschweig", "Munich", "Cologne", "Hamburg", _
                    "Bad Waldliesborn", "Bremen", "Encinitas", "Anaheim"}

        For i As Integer = 1 To Count
            Dim tmpLastName, tmpFirstName As String
            tmpLastName = tmpLastNames(tmpRandom.Next(tmpLastNames.Length - 1))
            tmpFirstName = tmpFirstNames(tmpRandom.Next(tmpLastNames.Length - 1))
            tmpListOfAddresses.Add(New Address( _
                            i.ToString, _
                            tmpLastName, _
                            tmpFirstName, _
                            tmpStreetNames(tmpRandom.Next(tmpStreetNames.Length - 1)), _
                            tmpRandom.Next(99999).ToString("00000"), _
                            tmpCities(tmpRandom.Next(tmpCities.Length - 1))))
        Next
        Return tmpListOfAddresses
    End Function
End Class

<Serializable()>
Public Class Address

    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    ''' <remarks>The default constructor which doesn't take any arguments is necessary 
    ''' to enable this class for serialization.</remarks>
    Sub New()
        MyBase.New()
    End Sub

    Sub New(ByVal Matchcode As String, ByVal Name As String, ByVal FirstName As String, _
            ByVal Street As String, ByVal Zip As String, ByVal Town As String)
        Me.Matchcode = Matchcode
        Me.LastName = Name
        Me.FirstName = FirstName
        Me.Street = Street
        Me.Zip = Zip
        Me.City = Town
    End Sub

    Public Overridable Property Matchcode() As String
    Public Overridable Property LastName() As String
    Public Overridable Property FirstName() As String
    Public Overridable Property Street() As String
    Public Overridable Property Zip() As String
    Public Overridable Property City() As String

    Public Overrides Function ToString() As String
        Return Matchcode & ":" & LastName & ", " & FirstName
    End Function
End Class
