Public Class frmMain

    Private myPrimes As List(Of Long)

    'Demonstrates the execution of a long running task as seperate thread so that the UI stays responsive.
    'Don't get the wrong idea, though: This doesn't parallalize the calculation of the primes.
    'The BackgroundWorkerComponent is not suitable for that!
    'Refer to the task examples in the book, instead.
    Private Sub btnStartCalculation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStartCalculation.Click

        'Define that the BackgroundWorker can inform about the Progress
        PrimesBackgroundWorker.WorkerReportsProgress = True

        'Configure progress bar.
        pbCalculationProgress.Minimum = 0
        pbCalculationProgress.Maximum = 1000
        pbCalculationProgress.Value = 0
        lblFoundText.Text = "The currently greatest found prime is:"

        'Trigger the event, so the Thread can be started in the DoWork event handler.
        'The argument for the greatest prime will be passes as arguemnt from the textbox.
        Me.PrimesBackgroundWorker.RunWorkerAsync(txtPrimeCount.Text)
    End Sub

    Private Sub PrimzahlenBackgroundWorker_DoWork(ByVal sender As Object, ByVal e As System.ComponentModel.DoWorkEventArgs) _
                                Handles PrimesBackgroundWorker.DoWork

        'Get the max prime count
        Dim locMax As Integer = CInt(e.Argument)

        Dim locProgressFactor As Double = 1000 / locMax
        Dim locProgressOld As Integer = 0
        Dim locProgressCurrent As Integer

        'Ready, if upper limit below 3.
        If (locMax < 3) Then Return

        'New array for taking the found primes.
        myPrimes = New List(Of Long)

        For z As Integer = 2 To locMax
            If IsPrime(myPrimes, z) Then
                myPrimes.Add(z)

                'Send the progress information. This cannot be happening too often,
                'because otherwise the UI thread becomes blocked.
                locProgressCurrent = CInt(z * locProgressFactor)
                If locProgressCurrent > locProgressOld Then
                    locProgressOld = locProgressCurrent
                    'Raise the progress event.
                    PrimesBackgroundWorker.ReportProgress(locProgressCurrent)
                End If
            End If
        Next
    End Sub

    Private Function IsPrime(ByVal primes As List(Of Long), ByVal Number As Long) As Boolean

        For Each testNumber As Long In primes
            If (Number Mod testNumber = 0) Then Return False
            If (testNumber >= Math.Sqrt(Number)) Then Exit For
        Next
        Return True
    End Function

    'Being called, when a BackgroundWorker component informs
    'about the progress. This event handler has automatically 
    'been marshalled to the UI process. You can access
    'controls of your form or User Control without any problems.
    Private Sub PrimesBackgroundWorker_ProgressChanged(ByVal sender As Object,
                                                       ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles PrimesBackgroundWorker.ProgressChanged
        pbCalculationProgress.Value = e.ProgressPercentage
        lblLargestPrime.Text = myPrimes(myPrimes.Count - 1).ToString("#,##0")
    End Sub

    'Being called, when the DoWork-Thread has finished.
    'This event handler also runs on the UI thread.
    Private Sub PrimesBackgroundWorker_RunWorkerCompleted(ByVal sender As Object,
                                                          ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles PrimesBackgroundWorker.RunWorkerCompleted
        lblFoundText.Text = "The largest found prime number is:"
        lblLargestPrime.Text = myPrimes(myPrimes.Count - 1).ToString("#,##0")
    End Sub

    Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
        Me.Close()
    End Sub
End Class
