﻿'Always remember to perform these imports
'when working with task and thread objects!
Imports System.Threading
Imports System.Threading.Tasks

Module Module1

    'Constant that determines the number of loops in the tasks.
    Private Const LOOPS As Integer = 8
    Private Const SPIN_ONE As Long = 8000000
    Private Const SPIN_TWO As Long = 10000000

    Sub Main()

        'Creates a task but doesn't start it.
        Dim firstTask As New Task(
            Sub()
                Console.WriteLine("Starting Task ID: " & Task.CurrentId.ToString)
                Console.WriteLine("Equals Thread ID: " &
                Thread.CurrentThread.ManagedThreadId.ToString)
                For count = 0 To LOOPS
                    Console.WriteLine(String.Format("ThreadId: {0:000} - Count equals: {1}",
                                      Thread.CurrentThread.ManagedThreadId,
                                      count))
                    'Simulate work load
                    Thread.SpinWait(8000000)
                Next
            End Sub)

        'Creates a task with the Factory class and starts the task.
        '(With the Factory class this cannot be done differently.)
        Dim taskByFactory = Task.Factory.StartNew(
            Sub()
                Console.WriteLine("Starting Task ID: " & Task.CurrentId.ToString)
                Console.WriteLine("Equals Thread ID: " &
                   Thread.CurrentThread.ManagedThreadId.ToString)
                For count = 0 To LOOPS
                    Console.WriteLine(String.Format(
                                      "ThreadId: {0:000} - Count equals: {1}",
                                      Thread.CurrentThread.ManagedThreadId,
                                      count))
                    'Simulate work load
                    Thread.SpinWait(10000000)
                Next
            End Sub)

        'The first task is started now:
        firstTask.Start()

        'If you press a key while the other tasks are still
        'running the application will still exit.
        Console.WriteLine("main thread is done and waiting!")
        Console.Read()

    End Sub

    Sub WaitingForTasks()

        Dim firstTask As New Task(
            Sub()

                Console.WriteLine("Starting Task ID: " & Task.CurrentId.ToString)
                Console.WriteLine("Equals Thread ID: " & Thread.CurrentThread.ManagedThreadId.ToString)
                For count = 0 To LOOPS
                    Console.WriteLine(String.Format("ThreadId: {0:000} - Count equals: {1}",
                                      Thread.CurrentThread.ManagedThreadId,
                                      count))
                    'Arbeitslast simulieren
                    Thread.SpinWait(SPIN_ONE)
                Next
            End Sub)

        'Creates a task with the Factory class and starts it.
        '(With the Factory class it's the only thing possible.)
        Dim taskByFactory = Task.Factory.StartNew(
            Sub()
                Console.WriteLine("Starting Task ID: " & Task.CurrentId.ToString)
                Console.WriteLine("Equals Thread ID: " & Thread.CurrentThread.ManagedThreadId.ToString)
                For count = 0 To LOOPS
                    Console.WriteLine(String.Format("ThreadId: {0:000} - Count equals: {1}",
                                      Thread.CurrentThread.ManagedThreadId,
                                      count))
                    'Arbeitslast simulieren
                    Thread.SpinWait(SPIN_TWO)
                Next
            End Sub)

        'The first task starts only now:
        firstTask.Start()

        'If you press a key while the other tasks are stilln
        'running the application will still exit.
        Console.WriteLine("Main thread has finished and is waiting for the results!")

        'Put the tasks the application should wait for into a task array
        '(The local type inference automatically creates a task array!)
        Dim tasksToWaitFor = {firstTask, taskByFactory}

        'Wait until the first task is finished:
        Dim taskFirstFinishedIndex = Task.WaitAny(tasksToWaitFor)

        'Identify what task is finished: the index is specified 
        'in the return value of WaitAny.
        Console.WriteLine(If(tasksToWaitFor(taskFirstFinishedIndex) Is taskByFactory,
                             "Factory task has finished first!",
                             "Manual task has finished first!"))

        'Wait until all (remaining) tasks are finished:
        Task.WaitAll(tasksToWaitFor)
        Console.WriteLine("All tasks have finished.")
        Console.Read()
    End Sub

    'Method that runs as a task and accepts a parameter:
    '(The method that was the Lambda statement before).
    Function WorkingProc(ByVal spinValue As Object) As Long
        Dim actualSpinValue = CInt(spinValue)
        Dim sw = Stopwatch.StartNew

        Console.WriteLine("Starting Task ID: " & Task.CurrentId.ToString)
        Console.WriteLine("Equals Thread ID: " & Thread.CurrentThread.ManagedThreadId.ToString)
        For count = 0 To LOOPS
            Console.WriteLine(String.Format("ThreadId: {0:000} - Count equals: {1}",
                              Thread.CurrentThread.ManagedThreadId,
                              count))
            'Simulate some workwork load
            Thread.SpinWait(actualSpinValue)
        Next

        sw.Stop()
        Return sw.ElapsedMilliseconds

    End Function

    Sub PassingReturningTaskValues()

        'Creates a task but doesn't start it.
        Dim firstTask As New Task(Of Long)(AddressOf WorkingProc, SPIN_ONE)

        'Creates a task with the Factory class and starts the task.
        Dim taskByFactory = Task.Factory.StartNew(Of Long)(AddressOf WorkingProc, SPIN_TWO)

        'The first task is started now:
        firstTask.Start()

        Console.WriteLine("Main thread is done and is waiting for results!")

        'Put the tasks the application should wait for into a task array
        '(The local type inference automatically creates a task array!)
        Dim tasksToWaitFor = {firstTask, taskByFactory}

        'Wait until the first task is finished:
        Dim taskFirstFinishedIndex = Task.WaitAny(tasksToWaitFor)

        'Identify what task is finished: the index is specified 
        'in the return value of WaitAny.
        Console.WriteLine(If(tasksToWaitFor(taskFirstFinishedIndex) Is taskByFactory,
                             "Factory Task wurde zuerst beendet nach ",
                             "Manueller Task wurde zuerst beendet nach ") &
                         tasksToWaitFor(taskFirstFinishedIndex).Result.ToString & " ms.")

        'Wait until all (remaining) tasks are finished:
        Task.WaitAll(tasksToWaitFor)
        Console.WriteLine("All tasks have finished.")
        Console.Read()

    End Sub

    Public Sub CallingWorkerThreadsWithMoreThanOneArgument()
        Dim var1 = "Testvar"
        Dim var2 = 42
        Dim var3 = 42.42
        Dim taskMoreParameters As New Task(Of Long)(
            Function() As Long
                Return WorkingProcMoreArguments(var1, var2, var3)
            End Function)
    End Sub

    Function WorkingProcMoreArguments(Arg1 As String, Arg2 As Integer, Arg3 As Double) As Long
        Return 42
    End Function


End Module
